
#ifndef __MCP2515_CAN_H__
#define __MCP2515_CAN_H__ 1

/**
 * \defgroup mcp2515_can_func CAN Nachrichten Handling
 *
 * \author 	Fabian Greif
 * \date	14.8.2005
 *
 * Copyright 2006 Fabian Greif
 * 
 * \version	$Id: mcp2515_can.h 2497 2005-11-19 21:45:49Z fabian $
 */
/*@{*/

#include "mcp.h"

/**
 * \brief	Datenstruktur zum Aufnehmen von CAN Nachrichten
 */
typedef struct
{
    uint16_t  id;			//!< Standard ID der Nachricht (11 Bits)
    uint8_t   rtr;			//!< Ist es ein Remote-Transmit-Request-Frame ?
    uint8_t   length;		//!< Anzahl der Datenbytes
    uint8_t   data[8];		//!< Die eigentlichen Daten der CAN Nachricht
} CANMessage;

/**
 * \brief	Verschickt eine Nachricht ber den CAN Bus
 *
 * \param	p_message	Nachricht die verschickt werden soll+
 * \return	0xff falls die Nachricht nicht verschickt werden konnte, \n
 *			ansonsten den Code des Puffes in den die Nachricht gespeichert wurde
 *
 */
extern uint8_t can_send_message(CANMessage *p_message);

/**
 * \brief	Liest eine Nachricht aus den Empfangspuffern des MCP2515
 *
 * \param	p_message	Pointer auf die Nachricht die gelesen werden soll
 * \return	Filtercode welcher die Nachricht akzeptiert hat.
 */
extern uint8_t can_get_message(CANMessage *p_message);

/*@}*/
#endif // __MCP2515_CAN_H__

